import { Database } from "bun:sqlite";
import { isNumber, isString } from "lodash";
import { z } from "zod";

const db = new Database("lab.sqlite", { strict: true })
initDB();


const boardSchema = z.object({
  id: z.number(),
  name: z.string(),
  room: z.string(),
  ipv4: z.string().ip({ version: "v4" }),
  ipv6: z.string().ip({ version: "v6" }),
  port: z.number().nonnegative(),
  cmdID: z.number().nonnegative()
}).partial({
  ipv6: true,
  cmdID: true
})

export type Board = z.infer<typeof boardSchema>

export function isBoard(obj: any): obj is Board {
  return boardSchema.safeParse(obj).success
}

const userSchema = z.object({
  id: z.number(),
  name: z.string(),
  password: z.string(),
  boardID: z.number(),
}).partial({
  boardID: true,
})

export type User = z.infer<typeof userSchema>

export function isUser(obj: any): obj is User {
  return userSchema.safeParse(obj).success
}


function initDB() {
  const tables = allTables()

  if (!tables.includes("Users")) {
    db.query(`
      CREATE TABLE Users(
        id         INT PRIMARY KEY   NOT NULL,
        name       TEXT              NOT NULL,
        password   TEXT              NOT NULL
      );
    `).run();
  }

  if (!tables.includes("Boards"))
    db.query(`
      CREATE TABLE Boards(
        id         INT PRIMARY KEY   NOT NULL,
        name       TEXT              NOT NULL,
        room       TEXT              NOT NULL,
        ipv4       CHAR(16)          NOT NULL,
        ipv6       CHAR(46)                  ,
        port       INT               NOT NULL
      )
    `).run();
}

export function allTables(): Array<string> {
  const query = db.query(`SELECT name FROM sqlite_master WHERE type='table'`)
  var tables = new Array()
  // Flaten array
  for (const item of query.values()) {
    tables.push(item[0])
  }

  query.finalize()
  return tables
}

export function tableColumnName(table: string): Array<string> {
  const query = db.query(`PRAGMA table_info(${table})`)

  var columnName = new Array()
  for (const column of query.values()) {
    columnName.push(column[1])
  }

  return columnName
}

export function allBoards() {
  const query = db.query(`SELECT * FROM Boards`)
  const boards = query.all()

  query.finalize()
  return boards
}

export function findBoard(id?: number | string): Board {
  let condition: string
  if (isNumber(id)) {
    condition = `id=${id}`
  } else if (isString(id)) {
    condition = `name='${id}'`
  } else {
    throw new Error("Failure: Wrong type when find board")
  }

  const query = db.query(`SELECT * FROM Boards WHERE ${condition}`)

  const spRet = boardSchema.safeParse(query.get())
  if (spRet.success) {
    return spRet.data
  } else {
    throw new Error(`Not Found ${id} FPGA Board`)
  }
}

export function findUser(name: string): User {
  const query = db.query(`SELECT * FROM Users WHERE name='${name}'`)

  const spRet = userSchema.safeParse(query.get())
  if (spRet.success) {
    return spRet.data
  } else {
    throw new Error(`Failure: Not found ${name} User`)
  }
}

